import { world } from "@minecraft/server";
import { Automata } from "../Automata";
import { PlayerSaveData } from "./PlayerSaveData";
import { AutomataSaveData } from "./AutomataSaveData";

const /**@type {Record<String,PlayerSaveData>} */ playerSaveDataRecord = {}
const /**@type {Record<String,Automata>} */publicAutomatasSaveRecord = {}

export class Database {

    static initialize(){
        console.warn(world.getDynamicPropertyTotalByteCount())
        for(const id of world.getDynamicPropertyIds()){
            let args = id.split(":");
            switch(args[0]){
                case 'save':
                    this.loadAutomataSave(args[2],args[1],JSON.parse(world.getDynamicProperty(id)));
                break;
            }
        }
    }

    /**
     * 
     * @param {String} name 
     * @param {Automata} automata 
     * @param {String} author 
     * @param {Boolean} isPrivate 
     * 
     * Guarda un automata con un nombre sobreescribiendo el anterior si es que existe
     * 
     */
    static saveAutomata(name,automata,author="system",isPrivate=false){
        let saveData = this.getPlayerSaveData(author);
        let oldSave = saveData.delete(name);
        if(oldSave && !oldSave.isPrivate) delete publicAutomatasSaveRecord[name+":"+author];
        saveData.save(name,isPrivate,automata.clone());
        if(!isPrivate) publicAutomatasSaveRecord[name+":"+author] = saveData.get(name);
    }

    static deleteAutomataSave(name,author="system"){
        this.getPlayerSaveData(author).delete(name);
    }

    /**
     * 
     * @param {String} name 
     * @param {String} author 
     * @param {Automata} automata 
     * @param {Boolean} isPrivate 
     * @returns 
     */
    static isAutomataSaveOverwriting(name,author,automata,isPrivate){
        return this.getPlayerSaveData(author).saveEquals(name,new AutomataSaveData(automata,author,isPrivate));
    }

    /**
     * 
     * @param {String} name 
     * @param {String} author 
     * @param {{
    *    p: number,
    *    a: {
    *        sx: number,sy: number,
    *        s: {
    *            n: string,
    *            t: {
    *                c: string,
    *                sf: string,
    *                p: Vector2[],
    *            }[],
    *            px: number,
    *            py: number,
    *        }[],
    *        i: number,f: number[],
    * }} data
    */
    static loadAutomataSave(name,author,data){
        let saveData = this.getPlayerSaveData(author);
        saveData.saveRecord[name] = new AutomataSaveData(Automata.getFromShortJSON(data.a),author,data.p == 1);
        if(!data.p) publicAutomatasSaveRecord[name+":"+author] = saveData.saveRecord[name];
    }

    static getAutomata(name,author="system"){
        return this.getPlayerSaveData(author).get(name).automata.clone();
    }

    static getPlayerSaveData(author="system"){
        if(!playerSaveDataRecord[author]){
            playerSaveDataRecord[author] = new PlayerSaveData(author);
        }
        return playerSaveDataRecord[author];
    }

    static setPlayerWallType(playerName,type){
        this.getPlayerSaveData(playerName).setWallType(type);
    }

    static setPlayerStateType(playerName,type){
        this.getPlayerSaveData(playerName).setStateType(type);
    }

    static getPlayerAllSavesNames(playerName){
        return Object.keys(this.getPlayerSaveData(playerName).saveRecord);
    }

    static getPublicAutomatas(){
        return publicAutomatasSaveRecord;
    }

}