import { world } from "@minecraft/server";
import { AutomataSaveData } from "./AutomataSaveData";

const DEFAULT_WALL_TYPE = "minecraft:iron_block";
const DEFAULT_STATE_TYPE = "minecraft:gold_block";

export class PlayerSaveData {
    /**@type {String}*/playerName;
    /**@type {Record<String,AutomataSaveData>}*/saveRecord = {};
    /**@type {String} */wallType;
    /**@type {String} */stateType;

    constructor(playerName){
        this.playerName = playerName;
        this.wallType = world.getDynamicProperty("playerSave:"+playerName+":wallType");
        if(!this.wallType) {
            this.wallType = DEFAULT_WALL_TYPE;
            world.setDynamicProperty("playerSave:"+playerName+":wallType",DEFAULT_WALL_TYPE);
        }
        this.stateType = world.getDynamicProperty("playerSave:"+playerName+":stateType");
        if(!this.stateType) {
            this.stateType = DEFAULT_STATE_TYPE;
            world.setDynamicProperty("playerSave:"+playerName+":stateType",DEFAULT_STATE_TYPE);
        }
    }

    save(name,isPrivate=false,automata){
        this.saveRecord[name] = new AutomataSaveData(automata,this.playerName,isPrivate);
        world.setDynamicProperty("save:"+this.playerName+":"+name,JSON.stringify(this.saveRecord[name].toShortJSON()))
    }

    get(name){
        return this.saveRecord[name];
    }

    setWallType(type){
        this.wallType = type;
        world.setDynamicProperty("playerSave:"+playerName+":wallType",type);
    }
    setStateType(type){
        this.stateType = type;
        world.setDynamicProperty("playerSave:"+playerName+":stateType",type);
    }

    contains(name){
        return this.get(name) ? true : false;
    }

    /**
     * 
     * @param {String} name 
     * @param {AutomataSaveData} otherSave 
     * @returns 
     */
    saveEquals(name,otherSave){
        let s = this.saveRecord[name];
        if(s) return s.equals(otherSave);
        return false;
    }

    delete(name){
        let old = this.get(name);
        if(old){
            delete this.saveRecord[name];
            world.setDynamicProperty("save:"+this.playerName+":"+name,undefined);
            return old;
        }
    }
}