import { system } from "@minecraft/server";
import { PhysicAut } from "./PhysicAut";
import { State } from "./State";
import { Transition } from "./Transition";

const /**@type {Record<Number,ReadStringTime}>} */autReadStringRecord = {};

const currentColor = "§l§9"
export class ReadStringTime{

    /**@type {PhysicAut}*/automata;
    currentIndex = 0;
    /**@type {State} */ currentState;
    /**@type {Transition} */ currentTransition;
    /**@type {String} */ string;
    /**@type {Number} */ runId;
    /**@type {Player[]} */ players;

    constructor(automata,initialState,string,ticks,players){
        this.automata = automata;
        this.currentState = initialState;
        this.string = string;
        this.players = players;
        this.runId = system.runInterval(()=>this.update(),ticks);
        this.currentState.entityName.nameTag = currentColor+this.currentState.name;
        autReadStringRecord[automata.id] = this;
    }
    update(){
        if(this.currentTransition){
            this.currentTransition.entityName.nameTag = this.currentTransition.color+this.currentTransition.character;
            this.currentState = this.currentTransition.endState;
            this.currentState.entityName.nameTag = currentColor+this.currentState.name;
            this.currentTransition = undefined;
        } else {
            let result = this.readStringStep();
            if(result > 0){
                ReadStringTime.stopRead(this.automata.id);
                for(let p of this.players) this.automata.readStringMessage({
                    flag: result-1,
                    state: this.currentState,
                    charIndex: this.currentIndex
                },this.string,p);
            } else ++this.currentIndex;
            
        }
    }

    /**
    * @returns {0|1|2|3}
    * 
    * Flags:
    * 
    *  0: Se logró transicionar a otro estado.
    * 
    *  1: el estado leyó correctamente la cadena.
    * 
    *  2: La cadena termina en un estado que no pertenece al conjunto de estados finales.
    * 
    *  3: El último estado en leer la cadena no puede avanzar.
    * 
    */
    readStringStep(){
        this.currentState.setEntityName(this.automata.getStateType(this.currentState));
        if(this.currentIndex == this.string.length)
            return this.automata.finalStates.includes(this.automata.states.indexOf(this.currentState)) ?
            1 : 2;
        let char = this.string.charAt(this.currentIndex);
        for(let transition of this.currentState.transitions) {
            if(transition.character == char) {
                this.currentTransition = transition;
                transition.entityName.nameTag = currentColor+transition.character;
                return 0;
            } 
        }
        return 3;
    }

    static stopRead(/**@type {Number} */ id){
        if(autReadStringRecord[id]){
            if(autReadStringRecord[id].currentTransition && autReadStringRecord[id].currentTransition.isValid)
                autReadStringRecord[id].currentTransition.entityName.nameTag = autReadStringRecord[id].currentTransition.color+
                autReadStringRecord[id].currentTransition.character;
            else if(autReadStringRecord[id].currentState.isValid)
                autReadStringRecord[id].currentState.setEntityName(autReadStringRecord[id].automata.getStateType(autReadStringRecord[id].currentState));
            system.clearRun(autReadStringRecord[id].runId);
            delete autReadStringRecord[id];
        }
    }
}