import { EntityEquippableComponent, EquipmentSlot, MolangVariableMap, Player, TicksPerSecond, system} from "@minecraft/server";
import { State } from "./State";
import { Vec } from "./Vec";
import { Transition } from "./Transition";

const /**@type {Record<String,SetTransition}>} */playerSetTransitionRecord = {};
const DEFAULT_PARTICLE_TYPE = "aut:transition_pos";
const MOLANG_MAP = new MolangVariableMap();
MOLANG_MAP.setFloat("time",Transition.getUpdateTick()/TicksPerSecond+0.05);
MOLANG_MAP.setFloat("size",Transition.getPositionSeparation());
MOLANG_MAP.setFloat("is_final",0);

export class SetTransition{
    /**@type {import("@minecraft/server").Vector3} */location;
    /**@type {import("@minecraft/server").Dimension} */dimension;
    /**@type {import("@minecraft/server").Vector2} */size;
    /**@type {'x'|'z'} */axis;
    /**@type {'x'|'z'} */axisI;
    /**@type {State} */state;
    /**@type {EntityEquippableComponent} */equipment;
    /**@type {Number} */runId;
    /**@type {import("@minecraft/server").Vector2[]} */positions = [];
    /**@type {Player} */player;
    /**@type {String} */particleType = DEFAULT_PARTICLE_TYPE;
    /**@type {Boolean} */ isValid = true;

    constructor(player,state,dimension,location,axis,axisI,size,side){
        this.dimension = dimension;
        this.location = location;
        this.axis = axis;
        this.axisI = axisI;
        this.size = size;
        this.side = side;

        this.state = state;
        playerSetTransitionRecord[player.id] = this;
        this.player = player;
        this.equipment = player.getComponent("equippable");
        this.runId = system.runInterval(()=>this.update(),Transition.getUpdateTick());
        
    }

    static get(player){
        return playerSetTransitionRecord[player.id];
    }

    static verifyRemovedState(/**@type {State}*/s){
        for(let playerId of Object.keys(playerSetTransitionRecord)){
            if(playerSetTransitionRecord[playerId].state.equals(s)){
                playerSetTransitionRecord[playerId].delete();
                break;
            }
        }
    }

    addPosition(/**@type {import("@minecraft/server").Vector2}*/pos){
        if(this.positions.length == 0){
            this.positions = Vec.lineVec2Between(this.state.position,pos,Transition.getPositionSeparation());
        } else {
            let resetPositions = false;
            let lastIndex = this.positions.length-1
            for(let i = 0;i<this.positions.length;++i){
                if(Vec.equals(this.positions[i],pos)){
                    this.positions.splice(i,this.positions.length-i);
                    lastIndex = this.positions.length-1;
                    resetPositions = true;
                    break;
                }
            }
            if(!resetPositions) {
                this.positions = this.positions.concat(Vec.lineVec2Between(pos,this.positions[lastIndex],Transition.getPositionSeparation()));
            }
        }
        this.positions.push(pos);
        /*let s = "";
        for(let p of this.positions){
            s+=JSON.stringify(p)+", "
        }
        console.warn(s)*/
    }

    update(){
        if(this.equipment.isValid()){
            let itemStack = this.equipment.getEquipment(EquipmentSlot.Mainhand)
            if(itemStack && itemStack.typeId == 'aut:set_transition'){
                let loc = {
                    x: 0,
                    y: 0,
                    z: 0
                }
                loc[this.axisI]+=this.side+0.5;
                for(const pos of this.positions){
                    loc[this.axis] = pos[this.axis]+0.5;
                    loc.y = pos.y+0.5;
                    this.dimension.spawnParticle(this.particleType,Vec.addVec(this.location,loc),MOLANG_MAP);
                }
            } else this.delete();
        } else this.delete();
    }

    delete(){
        system.clearRun(this.runId);
        delete playerSetTransitionRecord[this.player.id];
        this.isValid = false;
    }
}