import { Transition } from "./Transition";
import { Vec } from "./Vec";

const DEFAULT_INITIAL_KEYCOLOR = "§l§a"
const DEFAULT_FINAL_KEYCOLOR = "§l§c"
const DEFAULT_FI_KEYCOLOR = "§l§3" //Estado inicial y final a la vez
const DEFAULT_NORMAL_KEYCOLOR = "§f"

export class State {
    /**@type {String} */name;
    /**@type {Transition[]} */transitions = [];
    /**@type {import("@minecraft/server").Vector2} */position;
    /**@type {import("@minecraft/server").Entity} */entityName;
    isValid = true;

    constructor(name,position){
        this.position = position;
        this.name = name;
    }

    setName(name,type){
        this.name = name;
        if(this.entityName) this.setEntityName(type);
    }
    showName(location=undefined,dimension=undefined,/**@type {'final'|'initial'|'normal'|'fi'} */type="normal"){
        if(location){
            if(this.entityName){
                if(!(Vec.equals(location,this.entityName.location) && this.entityName.dimension.id == dimension.id))
                    this.entityName.teleport(location,{dimension:dimension});
                    this.setEntityName(type);
            } else {
                this.entityName = dimension.spawnEntity("aut:name",location);
                this.setEntityName(type);
            }
        } else if(this.entityName){
            this.entityName.remove();
            this.entityName = undefined;
        }
    }

    setEntityName(/**@type {'final'|'initial'|'normal'|'fi'} */type="normal"){
        if(this.entityName) switch(type){
            case "initial": 
                this.entityName.nameTag = DEFAULT_INITIAL_KEYCOLOR+this.name;
            break;
            case "final": 
                this.entityName.nameTag = DEFAULT_FINAL_KEYCOLOR+this.name;
            break;
            case "normal": 
                this.entityName.nameTag = DEFAULT_NORMAL_KEYCOLOR+this.name;
            break;
            case "fi": 
                this.entityName.nameTag = DEFAULT_FI_KEYCOLOR+this.name;
            break;
        }
    }

    getMidLocation(location,axis,axisI,side){
        let /**@type {import("@minecraft/server").Vector3} */loc = {};
        loc[axis] = location[axis] + this.position.x + 0.5;
        loc.y = location.y + this.position.y;
        loc[axisI] = location[axisI] + 0.5*side;
        return loc;
    }
    
    equals(/**@type {State} */s){
        return (this.name == s.name || Vec.equals(this.position,s.position));
    }
    toString(){
        return this.name;
    }
    toShortJSON(){
        let t = [];
        for(let transition of this.transitions) t.push(transition.toShortJSON());
        return {
            n: this.name,
            t: t,
            px: this.position.x,
            py: this.position.y
        }
    }
}