import { State } from "./State";

const DEFAULT_TRANSITION_CHAR_COLOR = "§l§6";
const DEFAULT_TRANSITION_POSITION_SEPARATION = 0.25;
const DEFAULT_UPDATE_TICK = 5;

export class Transition {
    /**@type {State} */startState;
    /**@type {String} */character;
    /**@type {State} */endState;
    /**@type {import("@minecraft/server").Vector2[]} */unionPositions;
    /**@type {import("@minecraft/server").Entity} */entityName;
    color = DEFAULT_TRANSITION_CHAR_COLOR;
    isValid = true;

    constructor(startState,character,endState,unionPositions){
        this.startState = startState;
        this.character = character;
        this.endState = endState;
        this.unionPositions = unionPositions;
    }

    setChar(char){
        this.character = char;
        if(this.entityName) this.entityName.nameTag = this.color+this.character;
    }

    showChar(location=undefined,dimension=undefined){
        if(location){
            if(this.entityName){
                if(!(Vec.equals(location,this.entityName.location) && this.entityName.dimension.id == dimension.id)) {
                    this.entityName.teleport(location,{dimension:dimension});
                    this.entityName.nameTag = this.color+character;
                }
            } else {
                this.entityName = dimension.spawnEntity("aut:transition_name",location);
                this.entityName.nameTag = this.color+this.character;
            }
            return this.entityName.id;
        } else if(this.entityName){
            let entityId = this.entityName.id;
            this.entityName.remove();
            this.entityName = undefined;
            return entityId;
        }
        return undefined;
    }

    getMidLocation(location,axis,axisI,side){
        let midPos = this.unionPositions[Math.round((this.unionPositions.length-1)/2)];
        let /**@type {import("@minecraft/server").Vector3} */loc = {};
        loc[axis] = location[axis] + midPos.x + 0.5;
        loc.y = location.y + midPos.y+0.25;
        loc[axisI] = location[axisI] + 1*side;
        return loc;
    }

    toString(){
        return "("+this.startState.name+", "+this.character+") = "+this.endState.name;
    }

    toShortJSON(){
        return {
            c: this.character,
            sf: this.endState.name,
            p: this.unionPositions
        };
    }

    equals(t){
        return this.startState.equals(t.startState) && this.character == t.character;
    }

    static getPositionSeparation(){
        return DEFAULT_TRANSITION_POSITION_SEPARATION;
    }

    static getUpdateTick(){
        return DEFAULT_UPDATE_TICK;
    }
}