/**
 * Herramientas relacionadas con vectores, como sumar y calcular distancia
 */
export class Vec {
    static addScalar(vec,scalar){
        return {x:vec.x+scalar,y:vec.y+scalar,z:vec.z+scalar};
    }
    static addVec(vec1,vec2){
        return {x:vec1.x+vec2.x,y:vec1.y+vec2.y,z:vec1.z+vec2.z};
    }
    static addPosition(vec1,x,y,z){
        return {x:vec1.x+x,y:vec1.y+y,z:vec1.z+z};
    }
    static distance(vec1,vec2){
        let distance = Math.sqrt(Math.pow(vec2.x-vec1.x,2)+Math.pow(vec2.y-vec1.y,2))
        return Vec.isVec2(vec1) ? distance : Math.sqrt(distance*distance + Math.pow(vec2.z-vec1.z,2));
    }

    /**
     * 
     * @param {import("@minecraft/server").Vector2} vec1 
     * @param {import("@minecraft/server").Vector2} vec2 
     * @param {Number} increase 
     * @returns {import("@minecraft/server").Vector2[]}
     * 
     * Devuelve una linea de vectores presentes entre dos vectores, con una separación que depende del valor del "increase".
     * 
     */

    static lineVec2Between(vec1,vec2,increase=1){
        let dif = {
            x: vec2.x-vec1.x,
            y: vec2.y-vec1.y
        };
        let vecList = [];
        if(dif.x != 0){
            let ax = Math.abs(dif.x) >= Math.abs(dif.y) ? 'x' : 'y';
            let ay = ax == 'x' ? 'y' : 'x';
            let direction = dif[ax] > 0 ? 1 : -1;
            let m = dif[ay]/dif[ax];
            let v;
            for(let i = 0+increase;i<Math.abs(dif[ax]);i+=increase){
                v = vec1[ax]+i*direction;
                let newPos = {};
                newPos[ax] = v;
                newPos[ay] = m*(v-vec1[ax])+vec1[ay];
                vecList.push(newPos);
            }
        } else {
            let direction = dif.y > 0 ? 1 : -1;
            for(let i = 0+increase;i<Math.abs(dif.y);i+=increase){
                vecList.push({
                    x: vec2.x,
                    y: vec1.y+i*direction
                });
            }
        }
        return vecList;
    }

    static clone(vec){
        let newVec = {
            x: vec.x,
            y: vec.y
        }
        if(!this.isVec2(vec)) newVec.z = vec.z;
        return newVec;
    }

    static isVec2(vec){
        return vec.z == undefined;
    }

    static equals(vec1,vec2){
        let e = vec1.x == vec2.x && vec1.y == vec2.y;
        if(this.isVec2(vec1)) return e;
        return e && vec1.z == vec2.z;
    }
}